[llvm-commits] [llvm] r120104 - in /llvm/trunk: lib/MC/ELFObjectWriter.cpp test/MC/ELF/relocation-386.s

Rafael Espindola rafael.espindola at gmail.com
Wed Nov 24 11:23:51 PST 2010


Author: rafael
Date: Wed Nov 24 13:23:50 2010
New Revision: 120104

URL: http://llvm.org/viewvc/llvm-project?rev=120104&view=rev
Log:
Relocate with the symbol if the relocation is of kind NTPOFF.
Patch by David Meyer, I added the test.

Modified:
    llvm/trunk/lib/MC/ELFObjectWriter.cpp
    llvm/trunk/test/MC/ELF/relocation-386.s

Modified: llvm/trunk/lib/MC/ELFObjectWriter.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/MC/ELFObjectWriter.cpp?rev=120104&r1=120103&r2=120104&view=diff
==============================================================================
--- llvm/trunk/lib/MC/ELFObjectWriter.cpp (original)
+++ llvm/trunk/lib/MC/ELFObjectWriter.cpp Wed Nov 24 13:23:50 2010
@@ -729,7 +729,8 @@
   if (&Sec2 != &Section &&
       (Kind == MCSymbolRefExpr::VK_PLT ||
        Kind == MCSymbolRefExpr::VK_GOTPCREL ||
-       Kind == MCSymbolRefExpr::VK_GOTOFF)) {
+       Kind == MCSymbolRefExpr::VK_GOTOFF ||
+       Kind == MCSymbolRefExpr::VK_NTPOFF)) {
     if (Renamed)
       return Renamed;
     return &Symbol;

Modified: llvm/trunk/test/MC/ELF/relocation-386.s
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/test/MC/ELF/relocation-386.s?rev=120104&r1=120103&r2=120104&view=diff
==============================================================================
--- llvm/trunk/test/MC/ELF/relocation-386.s (original)
+++ llvm/trunk/test/MC/ELF/relocation-386.s Wed Nov 24 13:23:50 2010
@@ -1,7 +1,7 @@
 // RUN: llvm-mc -filetype=obj -triple i386-pc-linux-gnu %s -o - | elf-dump | FileCheck  %s
 
-// Test that we produce the correct relocation types and that the relocation
-// to .Lfoo uses the symbol and not the section.
+// Test that we produce the correct relocation types and that the relocations
+// correctly point to the section or the symbol.
 
 // Section 3 is bss
 // CHECK:      # Section 0x00000003
@@ -10,8 +10,18 @@
 // CHECK:      # Symbol 0x00000001
 // CHECK-NEXT: (('st_name', 0x00000005) # '.Lfoo'
 
-// Symbol 6 is section 3
-// CHECK:      # Symbol 0x00000006
+// Symbol 4 is zed
+// CHECK:      # Symbol 0x00000004
+// CHECK-NEXT: (('st_name', 0x00000035) # 'zed'
+// CHECK-NEXT:  ('st_value', 0x00000000)
+// CHECK-NEXT:  ('st_size', 0x00000000)
+// CHECK-NEXT:  ('st_bind', 0x00000000)
+// CHECK-NEXT:  ('st_type', 0x00000006)
+// CHECK-NEXT:  ('st_other', 0x00000000)
+// CHECK-NEXT:  ('st_shndx', 0x00000004)
+
+// Symbol 7 is section 3
+// CHECK:      # Symbol 0x00000007
 // CHECK-NEXT: (('st_name', 0x00000000) # ''
 // CHECK-NEXT:  ('st_value', 0x00000000)
 // CHECK-NEXT:  ('st_size', 0x00000000)
@@ -36,10 +46,10 @@
 // CHECK-NEXT:  ('r_type', 0x0000000a)
 // CHECK-NEXT: ),
 
-// Relocation 3 (bar3 at GOTOFF) is done with symbol 6 (bss)
+// Relocation 3 (bar3 at GOTOFF) is done with symbol 7 (bss)
 // CHECK-NEXT:  # Relocation 0x00000003
 // CHECK-NEXT: (('r_offset',
-// CHECK-NEXT:  ('r_sym', 0x00000006
+// CHECK-NEXT:  ('r_sym', 0x00000007
 // CHECK-NEXT:  ('r_type',
 // CHECK-NEXT: ),
 
@@ -53,49 +63,49 @@
 // Relocation 5 (foo at TLSGD) is of type R_386_TLS_GD
 // CHECK-NEXT: # Relocation 0x00000005
 // CHECK-NEXT: (('r_offset', 0x00000020)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000012)
 // CHECK-NEXT: ),
 
 // Relocation 6 ($foo at TPOFF) is of type R_386_TLS_LE_32
 // CHECK-NEXT: # Relocation 0x00000006
 // CHECK-NEXT: (('r_offset', 0x00000025)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000022)
 // CHECK-NEXT: ),
 
 // Relocation 7 (foo at INDNTPOFF) is of type R_386_TLS_IE
 // CHECK-NEXT: # Relocation 0x00000007
 // CHECK-NEXT: (('r_offset', 0x0000002b)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x0000000f)
 // CHECK-NEXT: ),
 
 // Relocation 8 (foo at NTPOFF) is of type R_386_TLS_LE
 // CHECK-NEXT: # Relocation 0x00000008
 // CHECK-NEXT: (('r_offset', 0x00000031)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000011)
 // CHECK-NEXT: ),
 
 // Relocation 9 (foo at GOTNTPOFF) is of type R_386_TLS_GOTIE
 // CHECK-NEXT: # Relocation 0x00000009
 // CHECK-NEXT: (('r_offset', 0x00000037)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000010)
 // CHECK-NEXT: ),
 
 // Relocation 10 (foo at TLSLDM) is of type R_386_TLS_LDM
 // CHECK-NEXT: # Relocation 0x0000000a
 // CHECK-NEXT: (('r_offset', 0x0000003d)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000013)
 // CHECK-NEXT: ),
 
 // Relocation 11 (foo at DTPOFF) is of type R_386_TLS_LDO_32
 // CHECK-NEXT: # Relocation 0x0000000b
 // CHECK-NEXT: (('r_offset', 0x00000043)
-// CHECK-NEXT:  ('r_sym', 0x0000000b)
+// CHECK-NEXT:  ('r_sym', 0x0000000d)
 // CHECK-NEXT:  ('r_type', 0x00000020)
 // CHECK-NEXT: ),
 // Relocation 12 (calll 4096) is of type R_386_PC32
@@ -104,6 +114,12 @@
 // CHECK-NEXT:  ('r_sym', 0x00000000)
 // CHECK-NEXT:  ('r_type', 0x00000002)
 // CHECK-NEXT: ),
+// Relocation 13 (zed at NTPOFF) is of type R_386_TLS_LE and uses the symbol
+// CHECK-NEXT: # Relocation 0x0000000d
+// CHECK-NEXT: (('r_offset', 0x0000004e)
+// CHECK-NEXT:  ('r_sym', 0x00000004)
+// CHECK-NEXT:  ('r_type', 0x00000011)
+// CHECK-NEXT: ),
 
         .text
 bar:
@@ -129,6 +145,11 @@
         leal foo at TLSLDM(%ebx), %eax
         leal foo at DTPOFF(%eax), %edx
         calll 4096
+        movl zed at NTPOFF(%eax), %eax
+
+        .section        zedsec,"awT", at progbits
+zed:
+        .long 0
 
         .section	.rodata.str1.16,"aMS", at progbits,1
 .Lfoo:





More information about the llvm-commits mailing list